" Vim filetype plugin file
" Language:     Survex
" Maintainer:   Thomas Holder <thomasxholder@compuserve.de>
" 
" Copyright (C) 2006 Thomas Holder
" Distributed under the terms of the GNU General Public License v2
" 
" This plugin maps the following keys:
" F2:    Open survex manual and jump to command under cursor
" F5:    Process file with cavern
" F6:    View in Aven (after process)
" F7:    Jump to next process error (afer process)
" F8:    Jump to previous process error (after process)
" F9:    View *.err file (after process)
" Enter: Only in insert mode and only if cursor is at end of line:
"        Complete "*data normal" to "*data normal from to length clino compass"
"        Complete "*data passage" to "*data passage station left right up down"
"        If line looks like survey data, try to auto complete station names
" %:     If matchit plugin is installed, jump between *begin and *end
" 
" For using the manual function, this script needs to know where your survex
" text manual is installed.
" let s:manual = "@PKGDOCDIR@/manual.txt"
let s:manual = "/usr/share/doc/survex-1.1.11_pre060918/manual.txt.gz"


if exists("b:did_ftplugin")
  finish
endif
let b:did_ftplugin = 1

setlocal formatoptions=rqo
setlocal comments=:;
setlocal commentstring=;%s
setlocal include=^\\s*\\*include
setlocal includeexpr=substitute(v:fname,'$','.svx','')

setlocal tabstop=8
setlocal ignorecase
compiler survex

" maps for running programs
map <buffer> <F5> :write<CR>:make<CR>
map <buffer> <F6> :silent !aven %:r.3d &<CR>
map <buffer> <F7> :cnext<CR>
map <buffer> <F8> :cprev<CR>
map <buffer> <F9> :sview %:r.err<CR>
imap <buffer> <F5> <ESC><F5>
imap <buffer> <F6> <ESC><F6>
imap <buffer> <F7> <ESC><F7>
imap <buffer> <F8> <ESC><F8>
imap <buffer> <F9> <ESC><F9>

" increment station name if it ends with digit
function! SvxIncrementStation(station)
	let myOffset = match(a:station, '[0-9]\+$')
	if myOffset == -1
		" station does not end with digit
		return ''
	endif
	" split station in prefix and number part (prefix may be empty
	" string), increment number
	let myPrefix = strpart(a:station, 0, myOffset)
	let myDigits = strpart(a:station, myOffset)
	return myPrefix . (myDigits + 1)
endfunction

" map carriage return key
" autocomplete station names when typing data
" autocomplete "*data normal" and "*data passage"
function! SvxAutoCompleteStation()
	if col('.') != col('$')
		" not at end of line, do nothing
		return "\<CR>"
	endif

	" get current line without trailing comment
	let cline = substitute(getline('.'), ';.*', '', '')

	if cline =~ '^\s*\_[*]'
		" either command or empty line
		if cline =~ '^\s*\*data\s\+normal\s*$'
			" autocomplete normal (ARGE order, clino and compass are swaped)
			return " from to length clino compass"
		elseif cline =~ '^\s*\*data\s\+passage\s*$'
			" autocomplete passage line (LRUD data)
			return " station left right up down"
		endif
		" do nothing
		return "\<CR>"
	endif

	let statFrom = ''
	let statTo   = ''
	" search prev *data line
	let lnum = line('.') - 1
	while lnum > 0
		let dline = getline(lnum)
		if dline =~ '^\s*\*data\s\+\%(default\|[a-z]\+\s\+from\s\+to\)\>'
			" extract station name (second), try to increment it
			let statSkip = matchend(cline, '\S\+')
			let statFrom = matchstr(cline, '\S\+', statSkip)
			let statTo   = SvxIncrementStation(statFrom)
			break
		elseif dline =~ '^\s*\*data\s\+[a-z]\+\s\+station\>'
			" extract station name (first), try to increment it
			let statFrom = matchstr(cline, '\S\+')
			let statFrom = SvxIncrementStation(statFrom)
			break
		endif
		let lnum = lnum - 1
	endwhile

	if statFrom == ''
		" no *data found
		return "\<CR>"
	elseif statTo == ''
		let spaceFrom = matchstr(cline, '\>\s\+')
		return "\<CR>" . statFrom . spaceFrom
	else
		let spaceFrom = matchstr(cline, '\>\s\+')
		let spaceSkip = matchend(cline, '\>\s\+')
		let spaceTo   = matchstr(cline, '\>\s\+', spaceSkip)
		return "\<CR>" . statFrom . spaceFrom . statTo . spaceTo
	endif
endfunction
imap <buffer> <CR> <C-R>=SvxAutoCompleteStation()<CR>

" open text manual on F2 and jump to command under cursor
function! SvxHelp()
	let myword = toupper(expand("<cWORD>"))
	if myword !~# '^\*[A-Z]\+$'
		" no command under cursor, just open manual, no jump
		let search = 'silent\ 0'
	else
		let myword = strpart(myword, 1)
		let search = 'silent\ /^\\C' . myword . '$/'
	endif
	let mapF2  = 'map\ <buffer>\ <F2>\ :q<CR>'
	let moveUp = 'exe\ "normal\ z\\<CR>"'
	exe 'sview +' . search . '|' . mapF2 . '|' . moveUp . ' ' . s:manual
endfunction
nmap <buffer> <F2> :call SvxHelp()<CR>
imap <buffer> <F2> <ESC><F2>

" jump between *begin and *end with %, requires matchit plugin
" (load it with "runtime macros/matchit.vim")
if exists("loaded_matchit")
	let b:match_words = '\*begin:\*end'
endif

" vi:ts=2:sw=2
